package com.cusc.adas.v2x.utils;

import java.lang.reflect.Field;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.SortedMap;
import java.util.TreeMap;

import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;

public class Parse {
	
	public static <T> T parse(ByteBuf data, T t) throws InstantiationException, IllegalAccessException, NoSuchFieldException, SecurityException, ClassNotFoundException {
		
		Field [] fields =  t.getClass().getDeclaredFields();
		List<Field> fList = new ArrayList<>();
		SortedMap<Integer,Field> rstMap = new TreeMap<>();
		for(Field field : fields) {
			field.setAccessible(true);
			if (field.isAnnotationPresent(Order.class)) {
				 String fieldName = field.getName();
				 Order order = field.getAnnotation(Order.class);
				 

				 int num = order.value();
				 //基础数据类型 且 不是数组
				 if(field.getType().isPrimitive() && !field.getType().isArray()	) {
					// data.re
					// field.set(fList, rstMap);
					 readBuf(t,field,data);
				 }else if(field.getClass().isPrimitive() && field.getType().isArray()){
					// 基础数据类型 且 是数组 
				 }else if(List.class.isAssignableFrom(field.getType())) {
					// 集合类型 
					 //读取属性的注解是否引用其他字段的长度
					if( field.isAnnotationPresent(RefNumFlag.class)) {
						
						RefNumFlag refNumFlag = field.getAnnotation(RefNumFlag.class);
						 String  refField = refNumFlag.value();
						 Class<?> clazz1 = t.getClass();
						 Field nameField = clazz1.getDeclaredField(refField);
						 Type genericType = nameField.getGenericType();
						 nameField.setAccessible(true);
						 
						 if (genericType instanceof ParameterizedType) {
							 ParameterizedType pt = (ParameterizedType) genericType;
					         Type[] actualTypes = pt.getActualTypeArguments();
					         if (actualTypes.length > 0) {
					        	 actualTypes[0].getClass();
					        	 int len= (int) nameField.get(t);
								 for(int i=0;i<len;i++) {
									 actualTypes[0].getClass();
								 }
					         }
						 }
					}
						
				 }else if(field.getType()==String.class) {
					//类型为字符串 
					 readBuf(t,field,data);
				 }else {
					//其他对象类型 
					 Class<?> cls = Class.forName(field.getType().getName());
					 Object obj =  cls.newInstance();
					 
					 parse(data,  obj);
					 field.set(t, obj);
				 }
				 rstMap.put(num,field);
			}
		}
		return t;
	}
	
	/**
	 * 读取字节流
	 * @param t
	 * @param field
	 * @param data
	 * @throws IllegalArgumentException
	 * @throws IllegalAccessException
	 */
	public static void  readBuf(Object t, Field field,ByteBuf data ) throws IllegalArgumentException, IllegalAccessException {
		if(data.readerIndex()==data.maxCapacity() )
			return ;
		 FieldDef fieldDef =null;
		 if(field.isAnnotationPresent(FieldDef.class)) {
			 fieldDef = field.getAnnotation(FieldDef.class);
		 }
		
		Class<?> type = field.getType();
		
		if(type==String.class &&  fieldDef!=null &&fieldDef.type().equals("BYTE")) {
			byte[]by = new byte[fieldDef.length()];
			data.readBytes(by);
			String str = new String(by, StandardCharsets.UTF_8);
			field.set(t, str);
		}else if(type==short.class) {
			field.set(t, data.readUnsignedByte());
		}else if(type==int.class) {
			field.set(t, data.readUnsignedShort());
		}else if(type==long.class && fieldDef==null ) {
			field.set(t, data.readUnsignedInt());
		}else if(type==long.class && fieldDef!=null &&fieldDef.type().equals("TIMESTAMP") && fieldDef.length()==8 ) {
			field.set(t, data.readLong());
		}else if(type==long.class && fieldDef!=null &&fieldDef.type().equals("BYTE")  ) {
			field.set(t, data.readLong());
		}
	}
	public static byte[] longToBytes(long value) {
	    ByteBuffer buffer = ByteBuffer.allocate(8);
	    buffer.order(ByteOrder.BIG_ENDIAN); // 指定小端模式
	    buffer.putLong(value); // 将long值存入buffer
	    return buffer.array(); // 获取byte数组
	}
	public static byte[] longToBytes1(long value) {
	    ByteBuffer buffer = ByteBuffer.allocate(4);
	    buffer.order(ByteOrder.BIG_ENDIAN); // 指定小端模式
	    buffer.putLong(value); // 将long值存入buffer
	    return buffer.array(); // 获取byte数组
	}
	
	public static byte[] intToBytes(int value) {
	    ByteBuffer buffer = ByteBuffer.allocate(4);
	    buffer.order(ByteOrder.BIG_ENDIAN); // 指定小端模式
	    buffer.putLong(value); // 将long值存入buffer
	    return buffer.array(); // 获取byte数组
	}
	
	
	public static String byteToHexString(byte[] bytes) {
		StringBuilder sb = new StringBuilder();
	    for (byte b : bytes) {
	        sb.append(String.format("%02x", b));
	    }
	    return sb.toString();
		
		//return null;
//	    ByteBuffer buffer = ByteBuffer.allocate(8);
//	    buffer.order(ByteOrder.LITTLE_ENDIAN); // 指定小端模式
//	    buffer.putLong(value); // 将long值存入buffer
//	    return buffer.array(); // 获取byte数组
	}
	public static String long2Hex(long num, int len) {
	    String hex = Long.toHexString(num);
	    int padLength = len * 2;
	    if (hex.length() >= padLength) {
	        return hex.substring(hex.length() - padLength); // 超长时截断
	    }
	    return String.format("%0" + padLength + "d", 0).replace("0", "0") + hex; // 或直接格式化
	}
	
	
	public static  void  main(String [] args) throws InstantiationException, IllegalAccessException, NoSuchFieldException, SecurityException, ClassNotFoundException {
		
		
		String hexString = "F20000000015018cb864640000000000";
		byte[] bytes = HexStringToByteArray.hexStringToByteArray(hexString);
		ByteBuf byteBuf = Unpooled.buffer(16);
		long l = 500000; 
		         //3600000000
		        // 1164916946
		// byte []  bs  = Long.toHexString(l);
		String  s = long2Hex(l,4);
		//longToBytes1(l);
		int i = 990; 
		String hex = Integer.toHexString(i);
		
		String  vid = "T1023459";
		byte[] by1 = vid.getBytes();
		s =  byteToHexString(by1);
		
		System.out.println(s);
		
//		 
//		 
//		List<Byte> byteList = new ArrayList<Byte>();
//		
//		Byte[] bytes = byteList.toArray(new Byte[byteList.size()]);
		//byte[] bs = convoter(bytes);
		//byteBuf.writeBytes(bytes);
		//VehicleMessage  vehicleMessage = parse(byteBuf,new VehicleMessage()); 
		//System.out.print(vehicleMessage);
		
	}
	
	
	public static void convoterBean( Object source , Object target) throws IllegalArgumentException, IllegalAccessException, ClassNotFoundException, InstantiationException, NoSuchFieldException, SecurityException {
		Field [] fields =  source.getClass().getDeclaredFields();
		List<Field> fList = new ArrayList<>();
		SortedMap<Integer,Field> rstMap = new TreeMap<>();
		for(Field field : fields) {
			field.setAccessible(true);
			Object o= field.get(source);
			if(o==null) {
				continue;
			}
			
			OffsetDef offsetDef =null;
			 if(field.isAnnotationPresent(OffsetDef.class)) {
				 offsetDef = field.getAnnotation(OffsetDef.class);
			 }
			 //获取目标对象指定的属性
			 Field tField =  target.getClass().getDeclaredField(field.getName());
			 tField.setAccessible(true);
			if (field.isAnnotationPresent(Order.class)) {
				if(field.getType().isPrimitive() && !field.getType().isArray()	) {
					//如果属性添加了自定义注解 offset
					
					Class<?> type = field.getType();
					if(offsetDef!=null) {
						double  offset =offsetDef.value();
						int  type1 =offsetDef.type();
						int  minValidLength =offsetDef.minValidLength();
						
						//startindex
						
						if(type==byte.class) {
							tField.set(target, o);
						}else if(type==short.class) {
							
							if(minValidLength>0 ) {
								int  len = String.valueOf((Short)o).length();
								if(len<minValidLength) {
									int subtract = minValidLength-len;
									double f =(Short)o*Math.pow(10, subtract)*offset;
									tField.set(target, (float)f);
								}else {
									double f =(Short)o*offset;
									tField.set(target, (float)f);
								}
							}else {
								double f =(Short)o*offset;
								tField.set(target, (float)f);
							}
							
							
							
						}else if(type==int.class) {
							if(minValidLength>0 ) {
								int  len = String.valueOf((Integer)o).length();
								if(len<minValidLength) {
									int subtract = minValidLength-len;
									double f =(Integer)o*Math.pow(10, subtract)*offset;
									tField.set(target, (float)f);
								}else {
									double f =(Integer)o*offset;
									tField.set(target, (float)f);
								}
							}else {
								double f =(Integer)o*offset;
								tField.set(target, (float)f);
							}
						}else if(type==long.class) {
							if(minValidLength>0 ) {
								int  len = String.valueOf((Long)o).length();
								if(len<minValidLength) {
									int subtract = minValidLength-len;
									double f =(Long)o*Math.pow(10, subtract)*offset;
									tField.set(target, (float)f);
								}else {
									double f =(Long)o*offset;
									tField.set(target, (float)f);
								}
							}else {
								double d =(Long)o*offset;
								
								tField.set(target, d);
							}
						}
						
						
					}else {
						tField.set(target, o);
					}
					
				 }else if(field.getClass().isPrimitive() && field.getType().isArray()){
					// 基础数据类型 且 是数组 
				 }else if(List.class.isAssignableFrom(field.getType())) {
					// 集合类型 
					 //读取属性的注解是否引用其他字段的长度
					if( field.isAnnotationPresent(RefNumFlag.class)) {
						
//						RefNumFlag refNumFlag = field.getAnnotation(RefNumFlag.class);
//						 String  refField = refNumFlag.value();
//						 Class<?> clazz1 = t.getClass();
//						 Field nameField = clazz1.getDeclaredField(refField);
//						 Type genericType = nameField.getGenericType();
//						 nameField.setAccessible(true);
//						 
//						 if (genericType instanceof ParameterizedType) {
//							 ParameterizedType pt = (ParameterizedType) genericType;
//					         Type[] actualTypes = pt.getActualTypeArguments();
//					         if (actualTypes.length > 0) {
//					        	 actualTypes[0].getClass();
//					        	 int len= (int) nameField.get(t);
//								 for(int i=0;i<len;i++) {
//									 actualTypes[0].getClass();
//								 }
//					         }
//						 }
					}
						
				 }else if(field.getType()==String.class) {
					//类型为字符串 
//					 try {
//						String s = (String)o;
//						String s1 =  new String(s.getBytes());
						 tField.set(target, o);
//					 }catch(Exception e) {
//						 e.printStackTrace();
//					 }
				 }else {
					
					//其他对象类型 
					 Class<?> cls = Class.forName(tField.getType().getName());
					 Object obj =  cls.newInstance();
					 
					 convoterBean(o,  obj);
					 tField.set(target, obj);
				 }
			}
		}
	}
	
	public static byte[]  convoter(Byte[] bytes) {
		byte [] bs = new byte[bytes.length];
		int i=0;
		for(Byte b :bytes ) {
			bs[i++] = b;
		}
		return bs;
	}
	
}
